/**
 ** CubeTestASFacile
 ** Damien BUCHET
 ** http://www.damien-buchet.fr
 **
 ** Source cr�e pour le site Actionscript Facile http://www.actionscript-facile.com/
 **
 ** Ce script utilise les librairies / framework suivant : 
 ** Papervision 3D : http://code.google.com/p/papervision3d/
 ** GTween : http://gskinner.com/libraries/gtween/
 ** 
 ** Ce script cree un Cube 3D en utilisant le framework Papervision
 ** Nous rajoutons par dessus des evenements sur les touches du clavier qui ferront tourner le cube,
 ** au click sur une face du cube pour faire zoomer/dezoomer la camera,
 ** et un mouvement de camera en fonction de la position de la souris e l'ecran
 **/

package
{
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.ui.Keyboard;
	
	import com.gskinner.motion.GTween;
	import com.gskinner.motion.GTweener;
	import com.gskinner.motion.easing.Circular;
	import com.gskinner.motion.easing.Back;
	import com.gskinner.motion.easing.Quintic;
	
	import org.papervision3d.view.BasicView;
	import org.papervision3d.objects.primitives.Cube;
	import org.papervision3d.materials.utils.MaterialsList;
	import org.papervision3d.materials.ColorMaterial;
	import org.papervision3d.events.InteractiveScene3DEvent;
	import org.papervision3d.core.proto.MaterialObject3D;
	
	public class CubeTestASFacile extends BasicView
	{
		
		private var _cube:Cube;
		private var _materialsList:MaterialsList;
		private var _cameraTween:GTween;
		
		public function CubeTestASFacile()
		{	
		
			/* Definition des parametres de la scene 3D Papervision et de la camera */
			super(1000,580);
			camera.focus = 30;
			camera.z = -2500;
			// Rend la scene interactive
			viewport.interactive = true;
			
			addEventListener(Event.ADDED_TO_STAGE, startRub);
		}
	
		private function startRub(e:Event):void
		{	
			removeEventListener(Event.ADDED_TO_STAGE, startRub);
			
			/** 	Creation des materials (face) du cube 
			  *		On va simplement creer des faces de couleur
			 **/
			_materialsList = new MaterialsList();
			// Ajout des faces
			_materialsList.addMaterial(new ColorMaterial(0xFF0000), "top");
			_materialsList.addMaterial(new ColorMaterial(0x00FF00), "bottom");
			_materialsList.addMaterial(new ColorMaterial(0x0000FF), "left");
			_materialsList.addMaterial(new ColorMaterial(0xFFFF00), "right");
			_materialsList.addMaterial(new ColorMaterial(0xFF00FF), "front");
			_materialsList.addMaterial(new ColorMaterial(0x00FFFF), "back");
			
			// Creation du cube avec la liste des materials crees
			_cube = new Cube(_materialsList, 500, 500, 500, 4, 4, 4); // (liste materials, largeur, profondeur, hauteur, segmentsL, segmentsP, segmentH);
			
			// On va parcourir tous les Materials du cube, afin de les rendre interactives
			for each(var mat:MaterialObject3D in _cube.materials.materialsByName)
			{
				mat.interactive = true;
			}
			
			// Ajout d'un evenement de type "click" sur le cube
			_cube.addEventListener(InteractiveScene3DEvent.OBJECT_CLICK, clickEvent_faceCube);
			// Ajout du curseur "main" sur les face
			viewport.buttonMode = true;
			
			// Ajout du cube sur la scene
			scene.addChild(_cube);
			
			// Ajout des evenements clavier
			stage.addEventListener(KeyboardEvent.KEY_UP, keyUpEvent),
			
			// Lancement de la fonction de rendu
			startRendering();
			
			// Creation de la tween qui permettra de deplacer la camera suivant la position de la souris
			_cameraTween = new GTween(camera, 2.5,{x:stage.stageWidth/2,y:stage.stageHeight/2},{ease:Quintic.easeOut});
		}
		
		private function clickEvent_faceCube(e:InteractiveScene3DEvent):void
		{
			// Zoom / Dezoom lors du clic sur une face
			if (camera.z == -2500) GTweener.to(camera, 1, {z:-1500}, {ease:Circular.easeOut});
			else GTweener.to(camera, 1, {z:-2500}, {ease:Circular.easeOut});
		}
		
		private function keyUpEvent(e:KeyboardEvent):void
		{
			switch (e.keyCode)
			{
				case Keyboard.LEFT:
					GTweener.to(_cube, 1, {rotationY:_cube.rotationY+90}, {ease:Back.easeInOut});
					break;
				case Keyboard.RIGHT:
					GTweener.to(_cube, 1, {rotationY:_cube.rotationY-90}, {ease:Back.easeInOut});
					break;
				case Keyboard.UP:
					GTweener.to(_cube, 1, {rotationX:_cube.rotationX+90}, {ease:Back.easeInOut});
					break;
				case Keyboard.DOWN:
					GTweener.to(_cube, 1, {rotationX:_cube.rotationX-90}, {ease:Back.easeInOut});
					break;
				default:
					// nothing
					
			}
		}
		
		override protected function onRenderTick(e:Event=null):void
        {
			var tox:Number = -(stage.stageWidth/2-stage.mouseX);
			var toy:Number = (stage.stageHeight/2-stage.mouseY)*2;
			
			// Update et tween de la camera, en fonction de la position de la souris 
			_cameraTween.setValues({x:tox,y:toy});
			
			// Rendu de la scene
			renderer.renderScene(scene, camera, viewport); 
        }
		
		
	}
}