﻿/**
 ** CubeTestASFacile
 ** Damien BUCHET
 ** http://www.damien-buchet.fr
 **
 ** Source cree pour le site Actionscript Facile http://www.actionscript-facile.com/
 **
 ** Ce script utilise les librairies / framework suivant : 
 ** Papervision 3D : http://code.google.com/p/papervision3d/
 ** GTween : http://gskinner.com/libraries/gtween/
 ** 
 ** Ce script cree un Cube 3D en utilisant le framework Papervision
 ** Nous rajoutons par dessus des evenements sur les touches du clavier qui ferront tourner le cube,
 ** au click sur une face du cube pour faire zoomer/dezoomer la camera,
 ** et un mouvement de camera en fonction de la position de la souris e l'ecran
 **/

package
{

	import com.gskinner.motion.easing.Quintic;
	import com.gskinner.motion.easing.Back;
	import flash.ui.Keyboard;
	import com.gskinner.motion.easing.Circular;
	import com.gskinner.motion.GTweener;
	import org.papervision3d.events.InteractiveScene3DEvent;
	import flash.events.KeyboardEvent;
	import org.papervision3d.core.proto.MaterialObject3D;
	import org.papervision3d.materials.ColorMaterial;
	import org.papervision3d.view.BasicView;
	import com.gskinner.motion.GTween;
	import org.papervision3d.materials.utils.MaterialsList;
	import org.papervision3d.objects.primitives.Cube;
	import flash.events.Event;
	import com.demonsters.debugger.MonsterDebugger;


	/**
	 * La fameuse classe Main qui étend BasicView pour PaperVision 3D
	 * -default-frame-rate 40
	 * 
	 */
	public class Main extends BasicView
	{
		private var _cube:Cube;
		private var _materialsList:MaterialsList;
		private var _cameraTween:GTween;
		private var nKeyPress : uint;

		/**
		 * Constructeur.
		 * 
		 */
		public function Main()
		{
			// attend la fin de l'initialisation de la scène
			this.addEventListener(Event.ADDED_TO_STAGE, onReady, false, 0, true);
			
			/* Definition des parametres de la scene 3D Papervision et de la camera */
			super(1000,580);
			camera.focus = 30;
			camera.z = -2500;
			// Rend la scene interactive
			viewport.interactive = true;
		}

		/**
		 * Démarrage de l'application.
		 * définit le nombre d'images par secondes dans les paramètres de compilation
		 * -default-frame-rate 40
		 * 
		 */
		private function onReady(event : Event) : void
		{
			this.removeEventListener(Event.ADDED_TO_STAGE, onReady);

			// Init De MonsterDebugger
			MonsterDebugger.initialize(this);
			MonsterDebugger.trace(this, "Start Application");

			startRub(event);
		}
		
		private function startRub(e:Event):void
		{	
			removeEventListener(Event.ADDED_TO_STAGE, startRub);
			
			/** 	Creation des materials (face) du cube 
			  *		On va simplement creer des faces de couleur
			 **/
			_materialsList = new MaterialsList();
			// Ajout des faces
			_materialsList.addMaterial(new ColorMaterial(0xFF0000), "top");
			_materialsList.addMaterial(new ColorMaterial(0x00FF00), "bottom");
			_materialsList.addMaterial(new ColorMaterial(0x0000FF), "left");
			_materialsList.addMaterial(new ColorMaterial(0xFFFF00), "right");
			_materialsList.addMaterial(new ColorMaterial(0xFF00FF), "front");
			_materialsList.addMaterial(new ColorMaterial(0x00FFFF), "back");
			
			// Creation du cube avec la liste des materials crees
			_cube = new Cube(_materialsList, 350, 350, 350, 4, 4, 4); // (liste materials, largeur, profondeur, hauteur, segmentsL, segmentsP, segmentH);
			
			// On va parcourir tous les Materials du cube, afin de les rendre interactives
			for each(var mat:MaterialObject3D in _cube.materials.materialsByName)
				mat.interactive = true;
			
			// Ajout d'un evenement de type "click" sur le cube
			_cube.addEventListener(InteractiveScene3DEvent.OBJECT_CLICK, clickEvent_faceCube);
			
			viewport.buttonMode = true;// Ajout du curseur "main" sur les face
			
			scene.addChild(_cube);// Ajout du cube sur la scene
			
			stage.addEventListener(KeyboardEvent.KEY_UP, keyUpEvent);// Ajout des evenements clavier
			stage.addEventListener(KeyboardEvent.KEY_DOWN, keyDownEvent);// Ajout des evenements clavier
			
			startRendering();// Lancement de la fonction de rendu
			
			// Creation de la tween qui permettra de deplacer la camera suivant la position de la souris
			_cameraTween = new GTween(camera, 2.5,{x:stage.stageWidth/2,y:stage.stageHeight/2},{ease:Quintic.easeOut});
		}


		
		/**
		 * Appel lors du clique sur une face du cube.
		 */
		private function clickEvent_faceCube(e:InteractiveScene3DEvent):void
		{
			// Zoom / Dezoom lors du clic sur une face
			if (camera.z == -2500) GTweener.to(camera, 1, {z:-1500}, {ease:Circular.easeOut});
			else GTweener.to(camera, 1, {z:-2500}, {ease:Circular.easeOut});
		}
		
		
		/**
		 * Appel lors de l'appui sur une flèche directionnelle du clavier.
		 * 
		 */
		private function keyUpEvent(e:KeyboardEvent):void
		{
			stage.removeEventListener(Event.ENTER_FRAME, moveCube);// Ajout des evenements clavier
		}
		
		private function keyDownEvent(event : KeyboardEvent) : void
		{
			nKeyPress = event.keyCode;
			stage.addEventListener(Event.ENTER_FRAME, moveCube);// Ajout des evenements clavier
		}

		
		
		/**
		 * Déplace le cub en fonction de la touche appuyée du clavier.
		 * 
		 */
		private function moveCube(event : Event) : void
		{
			switch (nKeyPress)
			{
				case Keyboard.LEFT:
					GTweener.to(_cube, .1, {rotationY:_cube.rotationY+90}, {ease:Back.easeInOut});
					break;
					
				case Keyboard.RIGHT:
					GTweener.to(_cube, 0.1, {rotationY:_cube.rotationY-90}, {ease:Back.easeInOut});
					break;
					
				case Keyboard.UP:
					GTweener.to(_cube, 0.1, {rotationX:_cube.rotationX+90}, {ease:Back.easeInOut});
					break;
					
				case Keyboard.DOWN:
					GTweener.to(_cube, 0.1, {rotationX:_cube.rotationX-90}, {ease:Back.easeInOut});
					break;
					
				default:
					// nothing
					
			}
		}
		
		
		/**
		 * Déplacement de la caméra en fonction de la position de la souris.
		 * 
		 */
		override protected function onRenderTick(e:Event=null):void
        {
			var tox:Number = -(stage.stageWidth/2-stage.mouseX)*10;
			var toy:Number = (stage.stageHeight/2-stage.mouseY)*10;
			
			// Update et tween de la camera, en fonction de la position de la souris 
			_cameraTween.setValues({x:tox,y:toy});
			
			// Rendu de la scene
			renderer.renderScene(scene, camera, viewport); 
        }
		
		
	}
}
